/*
 * SOCKLAB - "laboratoire" d'etude des sockets internet - ENSIMAG - Juillet 93
 *
 * tools.c: outils divers (saisie de parametres...)
 * modified June 2000 by L. Bellissard
 *  approriate handling of big endian to little endian conversion for
 *  network names and network ports
 *
 */

/* modifie par Martin Heusse 25/02/2002 
 * utilisation de la librairie readline
 */

#include "socklab.h"
#include <stdlib.h>
#include <readline/readline.h>
#include <readline/history.h>

/*
// Cette fonction remplace l'appel a gets()
// qui est peu pratique, et de toute facon deconseille.
*/

rl_gets (char * str)
{
  char * line_read;

  /*  // une petite initialisation pas chere.
  //rl_set_prompt(" ");
  */

  /* Get a line from the user.  */
  line_read = readline(" ");

  /* et on la renvoit. */
  strcpy(str, line_read);
  /* et hop un peu de menage... */
  free(line_read);
}

/* Sinon ceci permet a priori de retrouver les bugs originaux :o)  /*

/*  char * rl_gets (char * line_read) */
/*  { */
/*    return gets(line_read); */
/*  } */


/* Obtention de l'adresse d'un host
 *=======================================================================
 *
 */

int host_addr(host, addr)
     char	host[];
     u_long	*addr;
{
  struct hostent	*hp;

  *addr = inet_addr(host);

  /*if (inet_aton(host,(struct in_addr*) addr)){*/
  if (*addr != INADDR_NONE){
    /* Cette @ est sous forme standard nbres + points */
    /* *addr = ntohl(*addr); /* inutile !!! */

    return(0);
  }
  else
    {
      /* elle est sous forme de chaine de caracteres. */

      hp = gethostbyname(host);
      if (hp == (struct hostent *)NULL)
	{
	  ERREUR("gethostbyname()");  /* !! code d'erreur... */
	  return(-1);
	}
      else
	{
	  
	  /* *addr = ntohl(*((u_long *)hp->h_addr)); /*inutile !! */
	  *addr = *((u_long *)hp->h_addr);
	  return(0);
	}
    }
}


/* Obtention d'un numero de port d'un service
 *====================================================================
 *
 */

int port_number(name, port)
	char	name[];
	int	*port;
{
	struct servent	*sp;
	
	if (sscanf(name, "%d", port) == 1) {
	  return(0);
	}
	else
	{
		sp = getservbyname(name,NULL);
		if (sp == (struct servent *)NULL)
		{
			ERREUR("getservbyname()");  /* !! code d'erreur */
			return(-1);
		}
		else
		{
			*port = ntohs(sp->s_port);
			return(0);
		}
	}
}



/* Saisie d'une adresse d'un host
 *=======================================================================
 * L'utilisateur peut donner un nom de machine ou une adresse brute (dot
 * notation. Si la machine est inconnue ou si le format d'adresse est
 * incorrect, la demande est reformulee. L'adresse est retournee par
 * parametre. Retourne -1 si l'utilisateur a annule, 0 sinon.
 */

int get_host(name, addr)
	char	name[];
	u_long	*addr;
{
	char	str[MAX_HOSTNAME];

	strcpy(str, name);
	for (;;)
	{
		if (strcmp(str,""))
			if (!strcmp(str,"*"))
			{
			        /* sert pour la creation d'une socket passive par exemple.*/
				*addr = INADDR_ANY;
				return;
			}
			else
				if (!strcmp(str, "."))
				{
				/*  // alias pour l'hote local*/
				/*	if (gethostname(str, MAX_HOSTNAME)==-1)
						ERREUR("gethostname()");
						else 
						if (host_addr(str, addr)==0)*/
				  if (host_addr("127.0.0.1", addr)==0)
				    return;
				}
				else /* c'est une machine quelconque */
					if (host_addr(str, addr)==0)
						return;

		printf("Host ?: ");
/*		printf("Host (\"*\"=INADDR_ANY, \".\"=<local host>) ?: ");*/
		rl_gets(str);
	}
}


/* Saisie d'un numero de port
 *=======================================================================
 * Retourne -1 si l'utilisateur a annule, 0 sinon.
 */

int get_port(name, port)
	char	name[];
	int	*port;
{
	char	str[MAX_PORTNAME+1];
	int	i;

	strcpy(str, name);
	for (;;)
	{
		if (strcmp(str,"") && port_number(str, port)==0)
			return;

		printf("Port ?: ");
		rl_gets(str);
	}

}



/* Saisie d'un message a envoyer
 *=======================================================================
 * L'utilisateur peut entrer un message precis ou juste donner sa taille
 * le raccourci "#n" ou  n est la taille desiree. Un message de n caracte-
 * res '*' est alors cree.
 */

int get_msg(msg, size)
	char	**msg;
	int	*size;
{
    int nb;

    for (;;)
        {
        if (strcmp(*msg,""))
            if (sscanf(*msg,"#%d", &nb)==1)
                {
		  if (nb>=0) /* attention : pas de limite de taille !!! */
                    {
                    free(*msg);
                    *msg = (char *)malloc((nb+1) * sizeof(char));
                    memset(*msg, '*', nb);
                    (*msg)[nb] = (char)0;
                    *size = nb;
                    return;
                    }
                else
                    printf("Max: %d octets\n", *size);
                }
                else
                    {
                    *size = strlen(*msg);
                    return;
                    }


                printf("Message ?: ", *size);
        rl_gets(*msg);
        }
}




/* Lecture d'un entier:
 *=========================================================================
 *
 *
 */

int get_nb(prompt, init, nb)
	char	prompt[];
	char	init[];
	int	*nb;
{
	int	defval;
	char	str[50];

	defval = *nb;
	if (!strcmp(init, ""))
		strcpy(str, "x");
	else
		strcpy(str, init);

	for (;;)
	{
		/* peut etre faut-il supprimer les eventuels blancs ? */
		if (!strcmp(str, ""))
			return;
		else
			if (sscanf(str, "%d", nb)==1)
				return;

		printf("%s (%d) ?: ", prompt, defval);

		rl_gets(str);
	}
}


/* Lecture d'un choix parmi n
 *===========================================================================
 *
 */

int get_choice(prompt, list, nb, init, choice)
	char	prompt[];
	t_item	list[];
	int	nb;
	char	init[];
	int	*choice;
{
	char	str[50];
	int	i;

	strcpy(str, init);
	for (;;)
	{
		if (strcmp(str,""))
			for (i=0; i<nb; i++)
				if (!strncmp(str, list[i].name, strlen(str)))
				{
					*choice = list[i].val;
					return;
				}

		printf("%s [", prompt);
		for (i=0; i<nb-1; i++)
			printf("%s/", list[i].name);
		printf("%s] ?: ", list[nb-1].name);
		rl_gets(str);
	}
}


/* Lecture d'une valeur booleenne (O/N)
 *========================================================================
 *
 */

int get_bool(prompt, var)
	char	prompt[];
	int	*var;
{
	char	str[50];

	*var = (*var ? 1 : 0);
	for (;;)
	{
		printf("%s (%c) ?: ", prompt, (*var ? 'Y' : 'N'));
		rl_gets(str);
		switch (my_toupper(str[0]))
		  {
		  case 0:	return;
		  case 'Y':	*var = 1; return;
		  case 'N':	*var = 0; return;
		  case 'O':     *var = 1; return; 
		  }
	}
}


/* saisie du numero de socket a traiter
 *=======================================================================
 *
 */

int get_id_sock(init, so)
	char	init[];
	int	*so;
{
	char	str[50];
	int	id;

	if (!strcmp(init, ""))
		strcpy(str, "x");
	else
		if (!strcmp(init, "."))
			sprintf(str, "%d", id_sock);
		else
			strcpy(str, init);

	for (;;)
	{
		if (!strcmp(str, ""))
			sprintf(str, "%d", id_sock);

		if (sscanf(str, "%d", &id)==1)
		{
			for (*so=0; *so<nbsock && sock[*so]!=id; (*so)++);
			if (*so<nbsock)
			{
				dft_sock = *so;
				return;
			}
		}
		
		
		if (dft_sock!=-1)
			printf("Id. socket (%d) ?: ", id_sock);
		else
			printf("Id socket ?: ");

		rl_gets(str);
	}
}


/* demande de confirmation
 *=======================================================================
 *
 */

int ask(prompt)
	char	prompt[];
{
	int	rep;

	rep = 0;
	get_bool(prompt, &rep);

	return(rep);
}


/* affichage de l'adresse d'une socket INTERNET:
 *=======================================================================
 * Utilisee uniquement pour debugger
 */

void display_inet_addr(addr)
	struct sockaddr_in addr;
{
	printf("     * Domaine: %d\n", (int)addr.sin_family);
	printf("     * Adresse: %s\n", inet_ntoa(addr.sin_addr));
	printf("     * Port:    %d\n", ntohs(addr.sin_port));
	printf("\n");
}


/* affichage d'un code d'erreur
 *=======================================================================
 * Appelee quand une primitive retourne une erreur (-1).
 * Le message est affiche en video inversee
 */

int ERREUR(msg)
	char	msg[];
{
	fprintf(stderr,"\07[7m");	/* Attention: non portable */
	perror(msg);
	fprintf(stderr,"[0m");
}


